<?php
/********************************************************************
  
   PhPeace - Portal Management System

   Copyright notice
   (C) 2003-2019 Francesco Iannuzzelli <francesco@phpeace.org>
   All rights reserved

   This script is part of PhPeace.
   PhPeace is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as 
   published by the Free Software Foundation; either version 2 of 
   the License, or (at your option) any later version.

   PhPeace is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   The GNU General Public License (GPL) is available at
   http://www.gnu.org/copyleft/gpl.html.
   A copy can be found in the file COPYING distributed with 
   these scripts.

   This copyright notice MUST APPEAR in all copies of the script!

********************************************************************/

include_once(SERVER_ROOT."/../classes/db.php");
include_once(SERVER_ROOT."/../classes/geo.php");
include_once(SERVER_ROOT."/../classes/ini.php");

/**
 * Manage generic calendar events functions
 *
 * @package PhPeace
 * @author Francesco Iannuzzelli <francesco@phpeace.org>
 */
class Events
{
	/** 
	 * @var Geo */
	public $geo;

	/**
	 * Topic ID
	 *
	 * @var integer
	 */
	public $id_topic;

	/**
	 * Topic group ID
	 *
	 * @var integer
	 */
	public $id_group;
	
	public $multiple_days;

	/**
	 * Initialize local properties
	 *
	 */
	function __construct()
	{
		$this->geo = new Geo();
		$this->id_topic = 0;
		$this->id_group = 0;
		$ini = new Ini;
		$this->multiple_days = $ini->GetModule("events","multiple_days",0);
	}

	/**
	 * Update calendar configuration
	 *
	 * @param string	$path			Calendar directory under the portal
	 * @param string	$title			Calendar name
	 * @param boolint	$events_insert	Whether visitors can insert events from the portal
	 * @param boolint	$search_events	Whether events are shown in search results
	 */
	public function ConfigurationUpdate($path,$title,$events_insert,$search_events,$multiple_days)
	{
		$ini = new Ini;
		$ini->Set("events_title",$title);
		$ini->SetPath("events_path",$path);
		$ini->Set("events_insert",$events_insert);
		$ini->Set("search_events",$search_events);
		$ini->SetModule("events","multiple_days",$multiple_days);
	}

	/**
	 * Total number of events
	 *
	 * @return integer
	 */
	public function Count()
	{
		$row = array();
		$db =& Db::globaldb();
		$db->query_single($row, "SELECT count(id_event) AS counter FROM events");
		return (int)$row['counter'];
	}

	/**
	 * Retrieve all approved events
	 *
	 * @return array
	 */
	public function EventsAll()
	{
		$sqlstr = "SELECT id_event,UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date
			FROM events WHERE approved=1";
		$rows = array();
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Retrieve all events in a specific day
	 *
	 * @param integer	$timestamp	Unix timestamp
	 * @return array
	 */
	public function EventsDay( $timestamp )
	{
		$date = date("Ymd",$timestamp);
		$join = $this->geo->GeoJoin("e.id_geo");
		$sqlstr = "SELECT id_event,e.title,e.description,place,{$join['name']} AS geo_name,type,allday,id_topic,
		UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,e.jump_to_article,e.id_article
		FROM events e {$join['join']}
		INNER JOIN event_types et ON e.id_event_type=et.id_event_type
		WHERE e.approved=1 ";
		if($this->id_topic>0)
			$sqlstr .=  " AND (e.id_topic='$this->id_topic' OR e.id_group='$this->id_group') ";
		else 
			$sqlstr .= " AND e.portal=1 ";
		if($this->multiple_days=="1")
			$sqlstr .= " AND UNIX_TIMESTAMP(DATE_FORMAT(e.start_date,'%Y%m%d'))<=UNIX_TIMESTAMP($date) 
				AND UNIX_TIMESTAMP(DATE_FORMAT(e.end_date,'%Y%m%d'))>=UNIX_TIMESTAMP($date) ";
		else 
			$sqlstr .= " AND DATE_FORMAT(e.start_date,'%Y%m%d')='$date' "; 
		$sqlstr .= " ORDER BY start_date ASC";
		$rows = array();
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Retrieve all events with a specific approved status
	 * (in the current topic)
	 *
	 * @param array		$rows		Events
	 * @param boolint 	$approved	Approved status
	 * @return integer				Num of events
	 */
	public function EventsApproved( &$rows, $approved)
	{
		$db =& Db::globaldb();
		$rows = array();
		$sqlstr = "SELECT UNIX_TIMESTAMP(start_date) AS start_date_ts,e.id_event,e.title,e.place,et.type,
		e.approved,IF(e.id_group>0 AND e.id_topic=0,tg.name,t.name) AS name
		FROM events e
		INNER JOIN event_types et USING (id_event_type)
		LEFT JOIN topics t ON e.id_topic=t.id_topic
		LEFT JOIN topics_groups tg ON e.id_group=tg.id_group
		WHERE e.approved=$approved ";
		if($this->id_topic>0)
			$sqlstr .= "  AND e.id_topic='$this->id_topic' ";
		$sqlstr .= " ORDER BY start_date DESC";
		return $db->QueryExe($rows, $sqlstr, true);
	}

	/**
	 * Retrieve all events for a specific month
	 *
	 * @param integer	$timestamp	Unix timestamp
	 * @return array
	 */
	public function EventsMonth( $timestamp )
	{
		if($timestamp==0)
			$timestamp = time();
		$year = date("Y",$timestamp);
		$month = date("m",$timestamp);
		$join = $this->geo->GeoJoin("e.id_geo");
		$rows = array();
		$sqlstr = "SELECT UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,
			id_event,e.title,place,type,{$join['name']} AS geo_name,
			e.id_topic,e.jump_to_article,e.id_article,DAYOFMONTH(start_date) AS dom 
		FROM events e 
		INNER JOIN event_types et ON e.id_event_type=et.id_event_type ";
		$sqlstr .= $join['join'];
		$sqlstr .= " WHERE e.approved=1 AND YEAR(e.start_date) = $year AND MONTH(e.start_date) = $month ";
		if($this->id_topic>0)
			$sqlstr .=  " AND e.id_topic='$this->id_topic' ";
		else
			$sqlstr .= " AND e.portal=1 ";
		$sqlstr .= " ORDER BY e.start_date ASC ";
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}
	
	/**
	 * Retrieve all events associated to a specific keyword
	 *
	 * @param integer $id_keyword	Keyword ID
	 * @param integer $limit		Limit number of results
	 * @param integer $year			Specific year (optional)
	 * @return array				Events
	 */
	public function Keyword($id_keyword,$limit,$year=0)
	{
		$join = $this->geo->GeoJoin("e.id_geo");
		$rows = array();
		if($year>0 && is_numeric($year))
			$sqlstr .= " AND YEAR(e.start_date)=$year ";
		$sqlstr = "SELECT UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,
			id_event,e.title,place,type,$join[name] AS geo_name,
			e.id_topic,'event' AS item_type,e.jump_to_article,e.id_article
			FROM keywords_use ku
			INNER JOIN events e ON ku.id=e.id_event AND ku.id_type=8 
			INNER JOIN event_types et ON e.id_event_type=et.id_event_type ";
		$sqlstr .= $join['join'];
		$sqlstr .= " WHERE ku.id_keyword=$id_keyword AND e.approved=1 AND " . ($this->multiple_days=="1"? "(e.start_date>=CURDATE() OR e.end_date>=CURDATE() )" : "e.start_date>=CURDATE()");
		if($this->id_topic>0)
			$sqlstr .=  " AND (e.id_topic='$this->id_topic' OR e.id_group='$this->id_group') ";
		else 
			$sqlstr .= " AND e.portal=1 ";
		$sqlstr .= " ORDER BY e.start_date ASC ";
		if($limit>0)
			$sqlstr .= " LIMIT $limit";
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}
	
	/**
	 * Retrieve all future events
	 *
	 * @param integer	$how_many	Limit how many events to return
	 * @param integer	$event_type	Event type ID
	 * @param boolean	$global		Whether events are from the global calendar or from a specific topic
	 * @param integer	$ts_limit	Date to filter events after (default to 0, i.e. today)
	 * @return array
	 */
	public function Next( $how_many, $event_type, $global, $ts_limit=0 )
	{
		$join = $this->geo->GeoJoin("e.id_geo");
		$rows = array();
		$sqlstr = "SELECT UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,
			id_event,e.title,place,type,{$join['name']} AS geo_name,
			e.id_topic,e.jump_to_article,e.id_article
		FROM events e 
		INNER JOIN event_types et ON e.id_event_type=et.id_event_type ";
		if($event_type>0)
			$sqlstr .= " AND e.id_event_type=$event_type ";
		$sqlstr .= $join['join'];
		$sqlstr .= " WHERE e.approved=1 AND " . ($this->multiple_days=="1"? "(e.start_date>=CURDATE() OR e.end_date>=CURDATE() )" : "e.start_date>=CURDATE()");
		if($global)
			$sqlstr .= " AND e.portal=1 ";
		elseif($this->id_topic>0)
		{
			if($this->id_group>0)
				$sqlstr .=  " AND (e.id_topic='$this->id_topic' OR e.id_group='$this->id_group') ";
			else
				$sqlstr .=  " AND e.id_topic='$this->id_topic' ";
		}
		if ($ts_limit>0)
			$sqlstr .= " AND UNIX_TIMESTAMP(e.start_date)<=$ts_limit ";
		$sqlstr .= " ORDER BY e.start_date ASC LIMIT $how_many";
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Return tha date range of all events in the calendar
	 *
	 * @return array	Containing two values, the minimum day and the maximum day (as Unix timestamps)
	 */
	public function Period()
	{
		$row = array();
		$db =& Db::globaldb();
		$db->query_single($row,	"SELECT UNIX_TIMESTAMP(MIN(start_date))-86400 AS min_ev_ts,UNIX_TIMESTAMP(MAX(start_date))+86400 AS max_ev_ts FROM events");
		return $row;
	}

	/**
	 * The recurring events for a specific date
	 *
	 * @param integer $ts	Unix timestamp of date
	 * @return array
	 */
	public function RecurringEvents( $ts )
	{
		$date = getdate($ts);
		$db =& Db::globaldb();
		$rows = array();
		$sqlstr = "SELECT id_event,r_day,r_month,r_year,description,url
			FROM recurring_events WHERE approved=1 AND r_day=$date[mday] AND r_month=$date[mon] ORDER BY r_year DESC";
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Retrieve recurring events (anniversaries)
	 *
	 * @param array 	$rows		Recurring events
	 * @param boolint 	$approved	Approved status
	 * @param integer 	$id_month	Specific month ID (optional)
	 * @return integer				Num of events
	 */
	public function Recurring( &$rows, $approved, $id_month=0 )
	{
		$db =& Db::globaldb();
		if ($id_month>0)
			$cond = " AND r_month=$id_month ";
		$rows = array();
		$sqlstr = "SELECT id_event,description,r_year,r_month,r_day
			FROM recurring_events WHERE approved=$approved $cond ORDER BY r_year DESC, r_month DESC, r_day DESC";
		return $db->QueryExe($rows, $sqlstr, true);
	}

	/**
	 * List of all months including approved recurring events
	 *
	 * @param array $rows	Months
	 * @return integer		Num of months
	 */
	public function RecurringMonths( &$rows )
	{
		$db =& Db::globaldb();
		$sqlstr = "SELECT r_month,count(id_event) AS num_events FROM recurring_events WHERE approved=1 GROUP BY r_month";
		return $db->QueryExe($rows, $sqlstr, true);
	}

	/**
	 * Search events
	 *
	 * @param array 	$rows			Events
	 * @param array 	$words			Search words
	 * @param array 	$params			Search params
	 * @param integer 	$time_weight	Weight to decrease ranking of past events (optional, default to 0)
	 * @param boolean 	$only_approved	Whether ony approved events should be returned (optional, default to true)
	 * @return integer					Num of events
	 */
	public function Search( &$rows,$words,$params,$time_weight=0,$only_approved=true )
	{
		$gjoin = $this->geo->GeoJoin("e.id_geo");
		$db =& Db::globaldb();
		$sqljoin = "";
		for ($i = 0; $i < count($words); $i++)
		{
			$sqljoin .= "INNER JOIN search_index2 si$i ON si$i.id = e.id_event AND si$i.id_res=8 ";
			if($this->id_topic>0)
				$sqlstr .=  " AND (si$i.id_topic='$this->id_topic' OR si$i.id_group='$this->id_group') ";
			$sqljoin .= "INNER JOIN search_words sw$i ON sw$i.id_word = si$i.id_word ";
			$wheres[] = "sw$i.word = '$words[$i]'";
			$scores[] = "SUM(si$i.score)";
		}
		if ($params['id_geo'] > 0)
			$cond .= " AND e.id_geo={$params['id_geo']}";
		if ($params['id_etype'] > 0)
			$cond .= " AND e.id_event_type={$params['id_etype']}";
		if($this->id_topic>0)
			$cond .=  " AND (e.id_topic='$this->id_topic' OR e.id_group='$this->id_group') ";
		if ($params['period'] > 0)
		{
			if ($params['period']==1)
				$cond .= " AND e.start_date>=CURDATE() ";
			if ($params['period']==2)
				$cond .= " AND e.start_date<CURDATE() ";
		}
		if ((strlen($params['start_date1']) > 2) && (strlen($params['start_date2']) > 2))
			$cond .= " AND e.start_date BETWEEN '{$params['start_date1']}' AND '{$params['start_date2']}'";
		if (strlen($params['keyword']) > 0)
		{
			include_once(SERVER_ROOT."/../classes/ontology.php");
			$o = new Ontology;
			$cond .= " AND k.keyword='{$params['keyword']}' ";
			$join = "INNER JOIN keywords_use ku ON ku.id=e.id_event AND ku.id_type=" . $o->types['event'] . " INNER JOIN keywords k ON ku.id_keyword=k.id_keyword";
		}
		$sqlstr = "SELECT UNIX_TIMESTAMP(e.start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,e.id_event,
			e.title,e.place,e.id_topic,e.jump_to_article,e.id_article,
			'event' AS item_type,evt.type,$gjoin[name] AS geo_name ";
		if(!$only_approved)
			$sqlstr .= ",e.approved";
		if(count($scores)>0)
		{
			$sqlstr .= ", ((" . implode("+",$scores) . ") ";
			if($time_weight>0)
				$sqlstr .= " - DATEDIFF(CURDATE(),e.start_date)*$time_weight ";
			$sqlstr .= " ) AS total_score ";
		}
		$sqlstr .= " FROM events e
		INNER JOIN event_types evt ON e.id_event_type=evt.id_event_type $gjoin[join] $sqljoin $join 
		WHERE 1=1 ";
		if ($only_approved)
			$sqlstr .= " AND  e.approved=1 ";
		if(count($wheres)>0)
			$sqlstr .= " AND  " . implode(" AND ", $wheres);
		$sqlstr .= " $cond GROUP BY e.id_event
		ORDER BY e.start_date DESC ";
		return $db->QueryExe($rows, $sqlstr,true);
	}

	/**
	 * Events submitted by a specific person
	 * (only approved ones)
	 *
	 * @param integer $id_p		Person ID
	 * @param integer $id_topic	Topic ID
	 * @return array			Events
	 */
	public function Submitted($id_p,$id_topic)
	{
		$cond = ($id_topic>0)? " AND e.id_topic=$id_topic" : "";
		$join = $this->geo->GeoJoin("e.id_geo");
		$rows = array();
		$sqlstr = "SELECT UNIX_TIMESTAMP(start_date) AS start_date_ts,UNIX_TIMESTAMP(end_date) AS end_date_ts,e.id_event,e.title,e.place,et.type,$join[name] AS geo_name,
			e.id_topic,e.jump_to_article,e.id_article,UNIX_TIMESTAMP(e.insert_date) AS hdate_ts
		FROM events e 
		INNER JOIN event_types et ON e.id_event_type=et.id_event_type ";
		$sqlstr .= $join['join'];
		$sqlstr .= " WHERE e.approved=1 $cond AND e.id_p=$id_p ORDER BY e.start_date DESC";
		$db =& Db::globaldb();
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Topics which have a subtopic of type calendar
	 * (i.e. can be shown in the topic assignment box for events)
	 *
	 * @return array
	 */
	public function Topics()
	{
		$rows = array();
		$db =& Db::globaldb();
		$sqlstr = "SELECT t.id_topic,t.name,t.id_group,tg.name as group_name
		FROM topics t 
		INNER JOIN topics_groups tg ON t.id_group=tg.id_group
		LEFT JOIN subtopics s ON t.id_topic=s.id_topic 
		WHERE s.id_type=10 GROUP BY t.id_topic
		ORDER BY tg.seq,tg.id_group,t.name";
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Types of events
	 *
	 * @return array
	 */
	public function Types()
	{
		$rows = array();
		$db =& Db::globaldb();
		$sqlstr = "SELECT id_event_type,type FROM event_types ORDER BY type";
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}

	/**
	 * Types of events (paginated)
	 *
	 * @param array $rows	Types
	 * @return integer		Num of types
	 */
	public function TypesP( &$rows )
	{
		$db =& Db::globaldb();
		$rows = array();
		$sqlstr = "SELECT id_event_type,type FROM event_types ORDER BY type";
		return $db->QueryExe($rows, $sqlstr, true);
	}

}
?>
